/*--------------------------------------------------------------------------------------------+
| Name : decode.c                                                                             |
| Purpose:                                                                                    |
| The file provide the functions for the convertion of the keyboard scancode into ascii code. |
| Before send the code to the convertion function, it is necessary to determine which one is  |
| vaild code and which one is not, a function isvalidcode() will help to do so. The scan code |
| like shift, caps etc will required to pass to the convertion function for the ascii code    |
| convertion. The return value form scancode2ascii(), however is incorrect, it is necessary to|
| hide the key, so isdispkey() will implemented, becasue the usage of printf(ascii).          |
+--------------------------------------------------------------------------------------------*/
#include "config.h"
#include "scanCodes.h"

/*
	If scancode is a function key, the return value from scancode2ascii is wrong. so monitor 
	should not display it. 0 means should not display, and 1 means can display the key.
*/
bit isdispkey(unsigned char rcvdata)
{
	// caps,left shift, right shift, key release
	if(rcvdata ==0x58|rcvdata ==0x12 |rcvdata ==0x59 |rcvdata ==0xE0 |rcvdata ==0xF0)
		return 0;
	else
		return 1;
}

/*
	This function can determine whether the rcvdata from keyboard is a valid key or not. For a
	example, when key 'q' is pressed keyboard will return 15F015 to indicate the key pressed and
	then released, so only the 15F0 is a vaild key, and later 15 is invalid. Another example is
	the 'Home' key, the keyboard will return E06CE0F06C, E0 indicate the extend key. Also E06CE0F0 
	is valid code, and later 6C is invalid.
*/
bit isvalidcode(unsigned char rcvdata)
{
	static bit releasekey=0;
	if(rcvdata == 0xF0)					// when 0xF0, the following rcvdata is invalid 
	{
		releasekey = 1;
		return 1;
	}
	else								
	{									
		if(releasekey)					// if releasekey is true, the current rcvdate is invalid
		{
			releasekey=0;
			return 0;
		}
		else							// else the current rcvdata is valid
		{
			return 1;
		}
	}						
}

/*
	the scancode decoder for the ascii output. with memory for extend, shift and caps
	if fine scancode not match, a alert bell will sound.
*/
unsigned char scancode2Ascii(unsigned char sc)
{
	static bit extend = 0;
	static bit shift = 0;
	static bit caps  = 0;
	static unsigned char kbdata;
	unsigned int i;

		switch (sc)
		{
			// FUNCTION CODE...
			case    0xE0:							// if extended code
					extend = 1;
					break;
			case 	0xF0 :							// The key is released, no extended
					extend = 0;						// code
					break;
			case 	0x12 :							// Left SHIFT				
					shift ^=1;					
					break;
			case 	0x59 :							// Right SHIFT
					shift ^= 1;
					break;
			case    0x58:							// cap
					caps ^= 1;
					if(!caps)
						shift=0;
					if(caps)
						shift=1;
					break;
			// DATA CODE...
			default:
			{	
				// IF EXTENDED CODE IS IMPLEMENTED...
				if(extend)							
				{
					extend = 0;
					// do a table look-up		
					for(i = 0; extended[i][0]!=sc && extended[i][0]; i++);
						if (extended[i][0] == sc) {	// end look-up if end search
							kbdata = extended[i][1];
						}
						else						// not match return a alert bell
						{
							kbdata='\a';
						}
				}
				// IF NOT EXTENDED CODE...
				else
				{
					// IF NOT SHIFTED...
					if(!shift)							
					{ 	// do a table look-up			
						for(i = 0; unshifted[i][0]!=sc && unshifted[i][0]; i++);
						if (unshifted[i][0] == sc) {	// end look-up if end search
							kbdata = unshifted[i][1];
						}
						else							// not match return a alert bell
						{
							kbdata='\a';
						}
						if(caps)
						{
							shift=1;
						}
					}
					// IF SHIFTED...
					else 								
					{
						// do a table look-up		
						for(i = 0; shifted[i][0]!=sc && shifted[i][0]; i++);
						if (shifted[i][0] == sc) {		// end look-up if end search
							kbdata = shifted[i][1];
						}
						else							// not match return a alert bell
						{
							kbdata='\a';
						}
						if(!caps)
						{
							shift = 0;
						}
					}// end of !shiftIf
				}// end of extendIf
				break;
			}// end of default
		}// end of switch
		return kbdata;
}


